//
//  ViewController.swift
//  PrebidTest
//
//  Created by Olena Stepaniuk on 02.11.2022.
//

import UIKit
import PrebidMobile
import GoogleMobileAds

fileprivate let storedResponseDisplayBanner = "response-prebid-banner-320-50"
fileprivate let storedImpDisplayBanner = "imp-prebid-banner-320-50"
fileprivate let gamAdUnitDisplayBannerOriginal      = "/21808260008/prebid_demo_app_original_api_banner"

class BannerViewController: UIViewController, GADBannerViewDelegate {
    
    @IBOutlet weak var bannerView: UIView!
    private var gamBanner: GAMBannerView!
    private let gamRequest = GAMRequest()
    
    private var prebidAdUnit: AdUnit!
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        print(Prebid.shared.version)
        setupBannerAdUnit()
        setupGAMBanner(width: 320, height: 50,
                       adUnitId: gamAdUnitDisplayBannerOriginal)
        loadGAMBanner()
    }
    
    func setupPrebidServer(storedResponse: String) {
        Prebid.shared.storedAuctionResponse = storedResponse
    }

    func setupBannerAdUnit() {
        setupPrebidServer(storedResponse: storedResponseDisplayBanner)
        
        let bannerAdUnit = BannerAdUnit(configId: storedImpDisplayBanner,
                                        size: CGSize(width: 320, height: 50))
        
        let parameters = BannerParameters()

        parameters.api = [Signals.Api.MRAID_2]
        
        bannerAdUnit.parameters = parameters
        
        prebidAdUnit = bannerAdUnit
        
        bannerAdUnit.setAutoRefreshMillis(time: 30000)
    }
    
    func setupGAMBanner(width: Int, height: Int, adUnitId: String) {
        let customAdSize = GADAdSizeFromCGSize(CGSize(width: width, height: height))
        
        gamBanner = GAMBannerView(adSize: customAdSize)
        gamBanner.adUnitID = adUnitId

        bannerView.constraints.first { $0.firstAttribute == .width }?.constant = CGFloat(width)
        bannerView.constraints.first { $0.firstAttribute == .height }?.constant = CGFloat(height)
    }
    
    func loadGAMBanner() {
        print("Google Mobile Ads SDK version: \(GADMobileAds.sharedInstance().sdkVersion)")
        
        gamBanner.backgroundColor = .red
        gamBanner.rootViewController = self
        gamBanner.delegate = self
        
        bannerView.addSubview(gamBanner)
        prebidAdUnit.fetchDemand(adObject: self.gamRequest) { [weak self] (resultCode: ResultCode) in
            print("Prebid demand fetch for AdManager \(resultCode.name())")
            self?.gamBanner.load(self?.gamRequest)
        }
    }
    
    //MARK: - GADBannerViewDelegate
    
    func bannerViewDidReceiveAd(_ bannerView: GADBannerView) {
        print("adViewDidReceiveAd")
        
        AdViewUtils.findPrebidCreativeSize(bannerView,
                                           success: { (size) in
            guard let bannerView = bannerView as? GAMBannerView else {
                return
            }

            bannerView.resize(GADAdSizeFromCGSize(size))

        },
                                           failure: { (error) in
            print("error: \(error)")

        })

        self.bannerView.constraints.first { $0.firstAttribute == .width }?.constant = bannerView.adSize.size.width
        self.bannerView.constraints.first { $0.firstAttribute == .height }?.constant = bannerView.adSize.size.height
    }

    func bannerView(_ bannerView: GADBannerView, didFailToReceiveAdWithError error: Error) {
        print("adView:didFailToReceiveAdWithError: \(error.localizedDescription)")
    }
}

